#!/bin/sh

set -eu

get_ver()
{
	awk "/FB_$1/ { v=\$3; gsub(\"\\\"\", \"\", v); print v}" < src/jrd/build_no.h
}

FB_MAJOR=$( get_ver 'MAJOR_VER' )
FB_MINOR=$( get_ver 'MINOR_VER' )
FB_REV=$( get_ver 'REV_NO' )

FB_VERSION="${FB_MAJOR}.${FB_MINOR}.${FB_REV}"

FB_VER="${FB_MAJOR}.${FB_MINOR}"
FB="firebird$FB_VER"
FB_no_dots=$(echo "$FB" | sed -e 's/\.//g')
FBDIR="firebird/$FB_VER"
ULFB="usr/lib/$DEB_HOST_MULTIARCH/$FBDIR"
VAR="var/lib/$FBDIR"
CLIENT_SO_VER=2
UTIL_SO_VER=
ODS_VER=$(grep -E '^const \S+ ODS_CURRENT\b' src/jrd/ods.h | sed 's/.*ODS_CURRENT//; s/\;.*//')

if echo ${DEB_BUILD_OPTIONS:-} | egrep -q '\bnodoc\b'; then
    NODOC="nodoc"
else
    NODOC=
fi

dat() {
    _copy f "$@"
}

exe() {
    _copy e "$@"
}

_copy() {
    local type=$1
    local dest=$2
    shift
    shift

    local dir
    if [ "${dest}" = "${dest%/}" ]; then
        dir=$(dirname "$dest")
    else
        dir=${dest%/}
    fi

    mkdir -v -p "$dir"

    case "$type" in
        e*) mode="755" ;;
        f*) mode="644" ;;
        *) echo "Error: Wrong params for copy!"; exit 1;;
    esac

    install -v -m "$mode" "$@" "$dest"
}

# Helper function used both in -super and -classic
copy_utils()
{
    for s in gbak gfix gpre gsec gstat isql nbackup fbsvcmgr fbtracemgr ;
    do
        target="$s"
        if [ "$target" = gstat ];
        then
            target=fbstat
        elif [ "$target" = isql ];
        then
            target=isql-fb
        fi

        exe "$D/usr/lib/$FBDIR/bin/$target" "$S/bin/$s"
    done
}

COMMON_DOC="/usr/share/doc/$FB-common-doc"

doc_symlink() {
    local doc_root
    doc_root="debian/$P/usr/share/doc"
    [ -d "$doc_root" ] || mkdir -p "$doc_root"
    ln -s "$FB-common-doc" "$doc_root/$P"
}

#-server-core
make_server_core () {
    P="$FB-server-core"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    mkdir -p "$D/var/log/firebird"

    find "$S/plugins" -type f -name '*.so' -not -name '*_example.so'| \
        while read f; do
            f="${f#$S/}"
            dat "$D/$ULFB/plugins/" "$S/$f"
        done


    exe "$D/$ULFB/intl/" "$S/intl/libfbintl.so"

    ln -s "/etc/$FBDIR/fbintl.conf" "$D/$ULFB/intl/"
    for f in firebird plugins; do
        ln -s "/etc/$FBDIR/$f.conf" "$D/$ULFB/"
    done

    doc_symlink
}

#-server
make_server () {
    P="$FB-server"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    mkdir -p "$D/usr/lib/$FBDIR/bin" \
             "$D/etc/xinetd.d" "$D/etc/$FBDIR" "$D/$VAR/data" \
             "$D/$VAR/system" "$D/$VAR/backup" \
             "$D/$ULFB"

    exe "$D/usr/lib/$FBDIR/bin/"              \
        "$S/bin/firebird"

    exe "$D/usr/lib/$FBDIR/bin/"       \
        "$S/bin/fb_lock_print"

    dat "$D/etc/xinetd.d/$FB_no_dots" \
        "debian/$P.xinetd"

    dat "$D/etc/$FBDIR" \
        "$S/databases.conf" \
        "$S/fbtrace.conf"

    for f in databases fbtrace ; do
        ln -s "/etc/$FBDIR/$f.conf" "$D/$ULFB/"
    done

    ln -s "/var/lib/$FBDIR/system/security${FB_MAJOR}.fdb" "$D/$ULFB/"

    # fix databases.conf: Remove references to employee
    sed -i -e'/^# Example Database/,/^employee / d' \
        "$D/etc/$FBDIR/databases.conf"

    # security db SQL
    mkdir -p "$D/usr/share/$FBDIR"
    dat "$D/usr/share/$FBDIR/" "src/dbs/security.sql"

    touch "$D/$VAR/backup/no_empty"
    touch "$D/$VAR/data/no_empty"

    exe "$D/usr/lib/$FBDIR/bin/" "$S/bin/fbguard"

    # systemd support
    dat "$D/lib/systemd/system/firebird$FB_VER.service" gen/install/misc/firebird.service
    sed -i -f /dev/stdin "$D/lib/systemd/system/"*.service <<EOF
/^PIDFile/ d
s,var/run/firebird/,run/firebird$FB_VER/,g
s,^ExecStart=.\\+/fbguard,ExecStart=/usr/sbin/fbguard --fb-ver $FB_VER,
/^ExecStart/ a RuntimeDirectory=firebird$FB_VER
s, syslog.target , ,
s,\\bsyslog.target ,,
s, syslog.target\\b,,
EOF

    mkdir -p "$D/usr/lib/tmpfiles.d"
    echo "d /tmp/firebird 0770 firebird firebird -" > "$D/usr/lib/tmpfiles.d/firebird$FB_VER.conf"

    dat "$D/$COMMON_DOC/examples/" debian/reindex-db

    ln -s "/run/firebird$FB_VER/fb_guard" "$D/$ULFB/fb_guard"

    doc_symlink
}

#libfbclient
make_libfbclient () {
    P="libfbclient$CLIENT_SO_VER"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    mkdir -p "$D/$ULFB/lib"

    exe "$D/usr/lib/$DEB_HOST_MULTIARCH" "$S/lib/libfbclient.so.$FB_VERSION"
    ln -s "libfbclient.so.$FB_VERSION" \
        "$D/usr/lib/$DEB_HOST_MULTIARCH/libfbclient.so.$CLIENT_SO_VER"

    ln -s "/usr/lib/$DEB_HOST_MULTIARCH/libfbclient.so.$CLIENT_SO_VER" \
        "$D/$ULFB/lib/"

    for m in "$S"/*.msg;
    do
        dat "$D/$ULFB" "$m"
    done

    doc_symlink
}

#libib-util
make_libib_util () {
    P="libib-util$UTIL_SO_VER"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    mkdir -p "$D/$ULFB/lib"

    exe "$D/usr/lib/$DEB_HOST_MULTIARCH/" "$S/lib/libib_util.so"

    ln -s "/usr/lib/$DEB_HOST_MULTIARCH/libib_util.so" "$D/$ULFB/lib/"

    doc_symlink
}


#(version)-utils
make_utils_bin () {
    P="$FB-utils"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    mkdir -p "$D/$ULFB/bin"

    copy_utils

    # manpages
    if [ -z "$NODOC" ]; then
        for u in fbstat gbak gsec isql-fb gfix gpre nbackup fbsvcmgr \
                 fbtracemgr fb_lock_print;
        do
            dat "$D/usr/share/$FBDIR/man/man1/" "debian/man/$u.1"
            gzip -9 -n "$D/usr/share/$FBDIR/man/man1/$u.1"
        done
        for u in fbguard firebird ;
        do
            dat "$D/usr/share/$FBDIR/man/man8/" "debian/man/$u.8"
            gzip -9 -n "$D/usr/share/$FBDIR/man/man8/$u.8"
        done
    fi

    for u in fbstat gbak gsec isql-fb gfix gpre nbackup fbsvcmgr fbtracemgr fb_lock_print;
    do
        ln -s "/usr/share/$P/fb_bin_wrapper_$FB_VER" "$D/$ULFB/bin/$u"
    done

    for u in fbguard firebird;
    do
        ln -s "/usr/share/$P/fb_sbin_wrapper_$FB_VER" "$D/$ULFB/bin/$u"
    done

    doc_symlink
}

#firebird-utils
make_utils() {
    P="firebird-utils"
    echo "Creating $P content"
    D="debian/$P"

    mkdir -p "$D/usr/bin" "$D/usr/sbin"
    mkdir -p "$D/usr/share/$P"

    exe "$D/usr/share/$P/fb_wrapper" debian/fb_wrapper

    for v in 3.0 4.0; do
        for dir in bin sbin; do
            cat <<EOF > "$D/usr/share/$P/fb_${dir}_wrapper_$FB_VER"
#!/bin/sh
export FIREBIRD_VERSION=$FB_VER
exec /usr/$dir/\$(basename "\$0") "\$*"
EOF
            chmod 0755 "$D/usr/share/$P/fb_${dir}_wrapper_$FB_VER"
        done
    done

    for u in fbstat gbak gsec isql-fb gfix gpre nbackup fbsvcmgr fbtracemgr fb_lock_print;
    do
        ln -s "/usr/share/$P/fb_wrapper" "$D/usr/bin/$u"
    done

    for u in fbguard firebird;
    do
        ln -s "/usr/share/$P/fb_wrapper" "$D/usr/sbin/$u"
    done

    if [ -z "$NODOC" ]; then
        dh_installman -p "$P" "debian/man/fb_wrapper.1"
    fi

    doc_symlink
}

#-common
make_common () {
    P="$FB-common"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    # config
    dat "$D/etc/$FBDIR/" "$S/firebird.conf"
    sed -i -f /dev/stdin "$D/etc/$FBDIR/firebird.conf" <<EOF
/^#RemoteBindAddress/ a RemoteBindAddress = localhost
s|^#SecurityDatabase.*|SecurityDatabase = /$VAR/system/security${FB_MAJOR}.fdb|
s|^#Providers.*|Providers = Remote,Engine${ODS_VER},Loopback|
EOF

    sed "s,\$(this),\$(root)/intl," "$S/intl/fbintl.conf" > "$D/etc/$FBDIR/fbintl.conf"
    dat "$D/etc/$FBDIR/plugins.conf" "$S/plugins.conf"

    dat "$D/usr/share/$P/" debian/functions.sh
    exe "$D/usr/share/$FBDIR/" debian/configure-port

    doc_symlink
}

#-dev
make_dev () {
    P="firebird-dev"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    dat "$D/usr/include/" "$S/include/"*.h
    dat "$D/usr/include/firebird/" "$S/include/firebird"/*.h
    dat "$D/usr/include/firebird/" "$S/include/firebird"/*.idl
    dat "$D/usr/include/firebird/" "$S/include-gen/Firebird.pas"
    dat "$D/usr/bin/" "$S/bin/fb_config"

    mkdir -p "$D/$ULFB/lib"

    ln -s "libfbclient.so.$CLIENT_SO_VER" \
        "$D/usr/lib/$DEB_HOST_MULTIARCH/libfbclient.so"

    ln -s "/usr/lib/$DEB_HOST_MULTIARCH/libfbclient.so" "$D/$ULFB/lib/"

    ln -s /usr/include "$D/$ULFB/"

    if [ -n "$UTIL_SO_VER" ]; then
        ln -s "libib_utill.so.$UTIL_SO_VER" \
            "$D/usr/lib/$DEB_HOST_MULTIARCH/libib_util.so"
    fi

    if [ -z "$NODOC" ]; then
        dh_installman -p "$P" debian/man/fb_config.1
    fi

    doc_symlink
}


#-examples
make_examples () {
    P="$FB-examples"
    echo "Creating $P content"
    D="debian/$P"
    S=debian/firebird-image

    mkdir -p "$D/$COMMON_DOC"
    cp -r "$S/examples" "$D/$COMMON_DOC"
    rm -r "$D/$COMMON_DOC/examples/empbuild"

    (
        echo "-- This is a generated file"
        echo "set sql dialect 3;"
        echo "create database 'employee.fdb';"
        cat "examples/empbuild/empddl.sql"
        cat "examples/empbuild/indexoff.sql"
        cat "examples/empbuild/empdml.sql"
        cat "examples/empbuild/indexon.sql"
        debian/emp_data.pl --table job --key job_code,job_grade,job_country \
            --blob-col job_requirement \
            < "examples/empbuild/job.inp"
        debian/emp_data.pl --table project --key proj_id \
            --blob-col proj_desc \
            < "examples/empbuild/proj.inp"
    ) > "$D/$COMMON_DOC/examples/employee.sql";


    # remove empty directories
    find "$D/$COMMON_DOC/examples" -type d -print0 \
        | xargs -0 rmdir --ignore-fail-on-non-empty -p

    install -m 0644 \
        "debian/$P.README.Debian" \
        "$D/$COMMON_DOC/examples/README.Debian"

    doc_symlink
}

#-doc
make_doc () {
    P="$FB-doc"
    echo "Creating $P content"
    D="debian/$P/$COMMON_DOC/doc"
    S=doc

    mkdir -p "$D"

    if [ -z "$NODOC" ]; then
        cp -r "$S"/* "$D/"

        recode windows-1251..utf8 "$D/ods11-index-structure.html"
        sed -i -e 's/windows-1252/UTF-8/' "$D/ods11-index-structure.html"

        rm -r "$D/license"
    fi

    doc_symlink
}

#-common-doc
make_common_doc() {
    P="$FB-common-doc"
    echo "Creating $P content"
    mkdir -p "debian/$P/$COMMON_DOC"

    install -m 0644 \
        "debian/README.Debian" \
        "debian/$P/$COMMON_DOC/README.Debian"
}

umask 022
make_server_core
make_server
make_libfbclient
make_utils_bin
make_utils
make_libib_util
make_common
make_dev
make_examples
make_doc
make_common_doc
echo "Packages ready."
exit 0
